# Subject: Per workspace sockets/instances.
# Description: When run from the command line, make geany open a new file in
#  the currently running instance only if there is an instance in the current
#  workspace. If not, open a new instance for the current workspace.
# Author: Erik de Castro Lopo <erikd@mega-nerd.com>
# Last-Update: 2010-08-16

=== modified file 'src/socket.c'
Index: geany-bzr/src/socket.c
===================================================================
--- geany-bzr.orig/src/socket.c
+++ geany-bzr/src/socket.c
@@ -271,6 +271,7 @@
 		return -1;
 #else
 	gchar *display_name = gdk_get_display();
+	gint workspace = ui_get_current_workspace (display_name);
 	gchar *hostname = utils_get_hostname();
 	gchar *p;
 
@@ -284,8 +285,8 @@
 		*p = '_';
 
 	if (socket_info.file_name == NULL)
-		socket_info.file_name = g_strdup_printf("%s%cgeany_socket_%s_%s",
-			app->configdir, G_DIR_SEPARATOR, hostname, display_name);
+		socket_info.file_name = g_strdup_printf("%s%cgeany_socket_%s_%s_ws%d",
+			app->configdir, G_DIR_SEPARATOR, hostname, display_name, workspace);
 
 	g_free(display_name);
 	g_free(hostname);
Index: geany-bzr/src/ui_utils.c
===================================================================
--- geany-bzr.orig/src/ui_utils.c
+++ geany-bzr/src/ui_utils.c
@@ -31,6 +31,13 @@
 #include <ctype.h>
 #include <gdk/gdkkeysyms.h>
 
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/Xatom.h>
+#endif
+
 #include "ui_utils.h"
 #include "prefs.h"
 #include "sciwrappers.h"
@@ -2419,3 +2426,54 @@
 		g_signal_stop_emission_by_name(editable, "insert-text");
 }
 
+/**
+ * Get the current visible workspace number for the given display name.
+ *
+ * If the X11 window property isn't found, 0 (the first workspace)
+ * is returned.
+ *
+ * Has been tested and verified to work under GNOME and KDE. Assuming that
+ * most other X11 desktops will work the same. For non-X11 backends it returns
+ * a workspace number of 0.
+ *
+ * This code is a slightly modified version of code that was ripped from Gedit
+ * which ripped it from Galeon.
+ **/
+gint ui_get_current_workspace(const gchar *display_name)
+{
+#ifdef GDK_WINDOWING_X11
+	GdkScreen * screen = gdk_screen_get_default ();
+	GdkWindow* root_win = gdk_screen_get_root_window (screen);
+	GdkDisplay* display = gdk_display_open (display_name);
+
+	Atom type;
+	gint format;
+	gulong nitems;
+	gulong bytes_after;
+	unsigned *current_desktop;
+	int err, result;
+	gint ret = 0;
+
+	gdk_error_trap_push ();
+	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
+					gdk_x11_get_xatom_by_name_for_display (display, "_NET_CURRENT_DESKTOP"),
+					0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
+					&bytes_after, (gpointer) &current_desktop);
+	err = gdk_error_trap_pop ();
+
+	if (err == Success && result == Success)
+	{
+		if (type == XA_CARDINAL && format == 32 && nitems > 0)
+			ret = current_desktop[0];
+		XFree (current_desktop);
+	}
+
+	gdk_display_close (display);
+
+	return ret;
+#else
+	/* FIXME: on mac etc proably there are native APIs
+	 * to get the current workspace etc */
+	return 0;
+#endif
+}
Index: geany-bzr/src/ui_utils.h
===================================================================
--- geany-bzr.orig/src/ui_utils.h
+++ geany-bzr/src/ui_utils.h
@@ -318,4 +318,6 @@
 
 gint ui_get_gtk_settings_integer(const gchar *property_name, gint default_value);
 
+gint ui_get_current_workspace(const gchar *display_name);
+
 #endif
